////////////////////////////////////////
/**************************************/
/*                                    */
/* Michael Goldbuch; Jrgen Dffinger */
/*                                    */
/* Mikroprozessortechnik              */
/* Praktikum 3 / Aufgabe2             */
/* Version 1.0                        */
/* 2011-05-04                         */
/*                                    */
/**************************************/
////////////////////////////////////////



//	definitions for ATMega8
.include "m8def.inc"



/***************   Definitionen   ***************/

.def HSRAM  = R31 ; High-Byte Adresse im SRAM fr Zeichenkette
.def LSRAM  = R30 ; Low-Byte Adresse im SRAM fr Zeichenkette
.def char   = R16 ; Zwischenspeicher fr empfangenes Zeichen
.def tmp    = R17 ; temporres Register (Arbeitsregister)
.def cCount = R18 ; Anzahl empfangener Zeichen


/***************   Makros   ***************/

.macro RECEIVE

RECEIVE_1:

	//	Wait for data to be received
	sbis UCSRA, RXC
	rjmp RECEIVE_1
	
	//	Get and return received data from buffer
	in char, UDR

.endm



.macro TRANSMIT_INIT

	// Register sichern
	push tmp

	//	receive disable	and transmit enable
	in   tmp,UCSRB
	andi tmp,0b01101111
	ori  tmp,0b01001000
	out  UCSRB,tmp

	//	Zeichen in den buffer schreiben, damit RET nicht mit ausgegeben wird
	subi cCount,0x03

	//	z auf Anfangsadresse setzen
	clr HSRAM
	ldi LSRAM,$60

	//	Erstes Zeichen senden 
	TRANSMIT

	//	Register wiederherstellen
	pop tmp
.endm



.macro TRANSMIT

	//	Register sichern
	push char

	//	Zeichen aus dem Speicher auslesen
	ld char,z+

TRANSMIT_1:

	; Put data (r16) into buffer, sends the data
	sbis UCSRA,UDRE
	rjmp TRANSMIT_1

	//	Zeichen in den buffer schreiben
	out UDR,char

	// Zeichenzhler dekrementieren
	dec cCount

	//	Register wiederherstellen
	pop char

.endm



/***************   Interupts setzen   ***************/

// Interrupts
rjmp RESET ; Reset Handler
reti; rjmp EXT_INT0 ; IRQ0 Handler
reti; rjmp EXT_INT1 ; IRQ1 Handler
reti; rjmp TIM2_COMP ; Timer2 Compare Handler
reti; rjmp TIM2_OVF ; Timer2 Overflow Handler
reti; rjmp TIM1_CAPT ; Timer1 Capture Handler
reti; rjmp TIM1_COMPA ; Timer1 CompareA Handler
reti; rjmp TIM1_COMPB ; Timer1 CompareB Handler
reti; rjmp TIM1_OVF ; Timer1 Overflow Handler
reti; rjmp TIM0_OVF ; Timer0 Overflow Handler
reti; rjmp SPI_STC ; SPI Transfer Complete Handler
rjmp USART_RXC ; USART RX Complete Handler
reti; rjmp USART_UDRE ; UDR Empty Handler
rjmp USART_TXC ; USART TX Complete Handler
reti; rjmp ADC ; ADC Conversion Complete Handler
reti; rjmp EE_RDY ; EEPROM Ready Handler
reti; rjmp ANA_COMP ; Analog Comparator Handler
reti; rjmp TWSI ; Two-wire Serial Interface Handler
reti; rjmp SPM_RDY ; Store Program Memory Ready Handler



/***************   RESET   ***************/

RESET:

	// 	Stack initialisieren
	ldi tmp,high(RAMEND)
	out sph,tmp			
	ldi tmp,low(RAMEND)	
	out spl,tmp		

	//	Speicheradresse (z) initialisieren
	clr HSRAM
	ldi LSRAM,$60

	//	Zeichenzhler auf 0 setzen
	clr cCount

	// 	USART initialisieren

	; Set baud rate (2400 BAUD)
	clr R17
	ldi R16,0xD0
	out UBRRH, R17
	out UBRRL, R16

	; Enable receiver and transmitter and the interupts
	ldi tmp, (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE)
	out UCSRB,tmp
	
	; Set frame format: 8data, 2stop bit
	ldi tmp, (1<<URSEL)|(1<<USBS)|(3<<UCSZ0)
	out UCSRC,tmp

	//	hintere LEDs initialisieren
	sbi DDRD,7
	sbi DDRC,0
	sbi DDRC,1
	cbi PORTD,7
	cbi PORTC,0
	cbi PORTC,1

	// global interupt enable
	sei

rjmp MAIN



/***************   Hauptprogramm   ***************/

MAIN:
	rjmp MAIN



/***************   USART_RXC   ***************/

USART_RXC:
	
	//	Resgister sichern
	push tmp
	in tmp,SREG
	push tmp

	// global Interupt disable
	cli

	//	Empfangenes Zeichen auslesen und in char speichern	
	RECEIVE 	

	// Empfangenes Zeichen in SRAM speichern
	st z+,char

	// Zeichenzhler inkrementieren
	inc cCount

	// Prfen ob mehr als drei Zeichen eingelesen wurden,
	// wenn ja dann Unterprogramm check_RET aufrufen, 
	//	um zu prfen ob RET gesendet wurde.
	mov tmp,cCount
	cpi tmp,0x03
	brge USART_RXC_1
	rjmp USART_RXC_2

USART_RXC_1:
	rcall check_RET

USART_RXC_2:

	//	global interupt enable
	sei

	//	Register wiederherstellen
	pop tmp
	out SREG,tmp
	pop tmp
reti



/***************   check_RET   ***************/

check_RET:
	
	//	Register sichern
	push tmp
	
	//	z sichern 
	push HSRAM
	push LSRAM
	
	//	z initialisieren
	subi LSRAM,0x03
	brcc check_RET_1
	dec HSRAM

check_RET_1:
	
	ld  tmp,z+
	cpi tmp,0x52
	brne check_RET_2
	ld 	tmp,z+
	cpi tmp,0x45
	brne check_RET_2
	ld  tmp,z+
	cpi tmp,0x54
	brne check_RET_2
	TRANSMIT_INIT
	pop tmp
	pop tmp
	rjmp check_RET_3

check_RET_2:
	
	//	z wiederhrstellen
	pop LSRAM
	pop HSRAM

check_RET_3:

	//	Register wiederherstellen
	pop tmp

ret



/***************   USART_TXC   ***************/

USART_TXC:
	
	//	Register sichern
	push tmp
	in   tmp,SREG
	push tmp

	//	Prfen ob noch ein Zeichen gesendet werden muss
	mov  tmp,cCount
	cpi  tmp,0x00
	breq USART_TXC_1
	
	//	nchstes Zeichen senden
	TRANSMIT
	rjmp USART_TXC_2

USART_TXC_1:

	//	receive enable and transmit disable
	in   tmp,UCSRB
	andi tmp,0b10110111
	ori  tmp,0b10010000
	out  UCSRB,tmp

	//	z auf Anfangsadresse (SRAM) setzen
	clr HSRAM
	ldi LSRAM,$60

USART_TXC_2:

	//	Register wiederherstellen
	pop tmp
	out SREG,tmp
	pop tmp

reti
